#*************************************************************************
#*
#*  $RCSfile: ChartDocument.tcl,v $
#*
#*  $Id: ChartDocument.tcl 230 2007-11-17 16:12:04Z grosserw $
#*
#*  last change: $Author: grosserw $ $Date: 2006-07-21 17:58:14 +0200 (Fr, 21 Jul 2006) $
#*
#*  The Contents of this file are made available subject to the terms of
#*  the following license
#*
#*  - BSD License
#*
#*  see file license.terms in this directory
#*
#*  Copyright 2005-2006 Wolfgang Grosser
#*
#*  Author: Wolfgang Grosser
#*
#*   All Rights Reserved.
#*
#*   Contributor(s): 
#*
#*
#************************************************************************/

itcl::class itcluno::ChartDocument {
	inherit OpenOffice

	protected variable data ""
	protected variable diagram ""

	private variable multiServiceFactory ""

	constructor {args} {eval OpenOffice::constructor unused $args} {}

	public method diagram {}
	public method serviceFactory {}

	public method setChartType {type}
	public method setData {dataList}
	public method setColumnDescriptions {headerList}
	public method setRowDescriptions {headerList}
	public method setTitle {title}
	public method setChart {columnHeaders rowHeaders data title type}
	public method setAreaProperties {propertyList}
	public method setDiagramProperties {propertyList}
	public method setDataPointProperties {col row propertyList}
	public method setDataRowProperties {row propertyList}

	public method getAreaProperties {{propertyList ""}}
	public method getDiagramProperties {{propertyList ""}}
	public method getDataPointProperties {col row {propertyList ""}}
	public method getDataRowProperties {row {propertyList ""}}

}
itcl::body itcluno::ChartDocument::constructor {args} {
	set document $handle
	set multiServiceFactory [itcluno::OfficeUtilities::queryInterface $document com.sun.star.lang.XMultiServiceFactory]
	set data [$document getData]
}
itcl::body itcluno::ChartDocument::serviceFactory {} {
	return $multiServiceFactory
}
itcl::body itcluno::ChartDocument::diagram {} {
	return $diagram
}
itcl::body itcluno::ChartDocument::setChartType {type} {
	set diagram [$multiServiceFactory createInstance "com.sun.star.chart.${type}Diagram"]
	$document setDiagram $diagram
}
itcl::body itcluno::ChartDocument::setData {dataList} {
	set rowSequence [list]
	set row 0
	foreach dataRow $dataList {
		lappend rowSequence [itcluno::OfficeUtilities::makeSequence $dataRow]
	}
	$data setData [itcluno::OfficeUtilities::makeSequence $rowSequence]
}
itcl::body itcluno::ChartDocument::setColumnDescriptions {headerList} {
	$data setColumnDescriptions [itcluno::OfficeUtilities::makeSequence $headerList]
}
itcl::body itcluno::ChartDocument::setRowDescriptions {headerList} {
	$data setRowDescriptions [itcluno::OfficeUtilities::makeSequence $headerList]
}
itcl::body itcluno::ChartDocument::setTitle {title} {
	set titleHandle [$document getTitle]
	setPropertyValues $titleHandle [list String $title]
}
itcl::body itcluno::ChartDocument::setChart {columnHeaders rowHeaders data title type} {
	if {[llength $columnHeaders] != [llength [lindex $data 0]]} {
		return -code error "column headers don't fit for data"
	}
	if {[llength $rowHeaders] != [llength $data]} {
		return -code error "row headers don't fit for data"
	}
	setData $data
	setColumnDescriptions $columnHeaders
	setRowDescriptions $rowHeaders
	setTitle $title
	setChartType $type
}
itcl::body itcluno::ChartDocument::setAreaProperties {propertyList} {
	set areaHandle [$document getArea]
	setPropertyValues $areaHandle $propertyList
}
itcl::body itcluno::ChartDocument::setDiagramProperties {propertyList} {
	setPropertyValues $diagram $propertyList
}
itcl::body itcluno::ChartDocument::setDataPointProperties {col row propertyList} {
	set properties [$diagram getDataPointProperties $col $row]
	setPropertyValues $properties $propertyList
}
itcl::body itcluno::ChartDocument::setDataRowProperties {row propertyList} {
	set properties [$diagram getDataRowProperties $row]
	setPropertyValues $properties $propertyList
}
itcl::body itcluno::ChartDocument::getAreaProperties {{propertyList ""}} {
	set areaHandle [$document getArea]
	return [getPropertyValues $areaHandle $propertyList]
}
itcl::body itcluno::ChartDocument::getDiagramProperties {{propertyList ""}} {
	return [getPropertyValues $diagram $propertyList]
}
itcl::body itcluno::ChartDocument::getDataRowProperties {row {propertyList ""}} {
	set properties [$diagram getDataRowProperties $row]
	#	here I need a way to find out all the available properties
	#	com.sun.star.drawing.FillProperties
	#	com.sun.star.drawing.LineProperties
	#	...
	return [getPropertyValues $properties $propertyList]
}
itcl::body itcluno::ChartDocument::getDataPointProperties {col row {propertyList ""}} {
	set properties [$diagram getDataPointProperties $col $row]
	#	here I need a way to find out all the available properties
	#	com.sun.star.drawing.FillProperties
	#	com.sun.star.drawing.LineProperties
	#	...
	return [getPropertyValues $properties $propertyList]
}
