#*************************************************************************
#*
#*  $RCSfile: Draw.tcl,v $
#*
#*  $Revision: 1.6 $
#*
#*  last change: $Author: grosserw $ $Date: 2006-01-30 08:59:31 +0100 (Mon, 30 Jan 2006) $
#*
#*  The Contents of this file are made available subject to the terms of
#*  the following license
#*
#*  - BSD License
#*
#*  see file license.terms in this directory
#*
#*  Copyright 2005-2006 Wolfgang Grosser
#*
#*  Author: Wolfgang Grosser
#*
#*   All Rights Reserved.
#*
#*   Contributor(s): 
#*
#*
#************************************************************************/

catch {package require Itcl}
package require tcluno_simpress

itcl::class itcluno::Draw {
	inherit OpenOffice

	protected variable drawPages ""
	protected variable _pageHandles
	protected variable _shapes

	protected variable masterPages ""
	protected variable masterPage

	protected variable chartIdx 0
	protected variable controller ""

	protected variable _presentationObject
	protected variable _objectId -1
	protected variable objectPrefix
	protected variable _bitmapId -1
	protected variable bitmapPrefix
	protected variable _textHandleArray
	protected variable _point ""

	constructor {args} {eval OpenOffice::constructor private:factory/sdraw $args} {}
	public method getPageNames {}
	public method insertPage {{position end} {layoutIndex -1}}
	public method renamePage {oldname newname}
	public method deletePage {pageName}
	public method activatePage {pageName}

	public method setMasterText {rectangle text {type Text}}
	public method setMasterImage {rectangle graphicURL}
	public method setText {pageName rectangle text {type Text}}
	public method setPreparedText {pageName index string {propertyList ""}}
	public method appendPreparedText {pageName index string {propertyList ""}}
	public method appendPreparedHyperLink {pageName index string {propertyList ""}}

	public method appendPreparedTextParagraph {pageName index string {propertyList ""}}
	public method setImage {pageName rectangle graphicURL}
	public method setPreparedImage {pageName index graphicURL}

	public method getDocumentProperties {{propertyList ""}}
	public method getMasterPageProperties {{propertyList ""}}
	public method getPageProperties {pageName {propertyList ""}}
	public method getObjectProperties {objectId {propertyList ""}}
	public method getPreparedObjectProperties {pageName index {propertyList ""}}
	public method setPagePropertyValues {pageName propertyList}
	public method setPropertyValues {objectId propertyList}
	public method setPreparedObjectProperties {pageName index propertyList}

	public method createTable {pageName rectangle}
	public method createChart {pageName rectangle}
	public method createShape {pageName rectangle shapeType {propertyList ""}}

	public method group {pageName shapeList name}

	public method exportToPdf {filename}

	protected method checkPageName {pageName}

	protected method getPageByName {pageName}
	protected method getPageByIndex {index}
	protected method checkForFilter {filename filterSequence}

	protected method createOLE2Object {pageName rectangle type}

	protected method _setImage {pageHandle rectangle graphicURL}

	private method _getController {}
	private method setFilterList {}
}

itcl::body itcluno::Draw::constructor {args} {
	setFilterList
	set objectPrefix "_${this}_@@@_"
	set bitmapPrefix "_${this}_bitmap_@@@_"
	if {[string length $filename] == 0} {
		set filename $factory
	}
	if {[catch {checkForFilter $filename $filterSequence} msg]} {
		puts stderr $msg
		#	try opening but without guarantee what happens
		openFile $filename $filterSequence
		set multiServiceFactory [itcluno::OfficeUtilities::queryInterface $document com.sun.star.lang.XMultiServiceFactory]
		return
	}
	openFile $filename $filterSequence
	set xDPS [itcluno::OfficeUtilities::queryInterface $document com.sun.star.drawing.XDrawPagesSupplier]
	set drawPages [$xDPS getDrawPages]
#	set drawPages [$document getDrawPages]
	set masterPages [$document getMasterPages]
	set multiServiceFactory [itcluno::OfficeUtilities::queryInterface $document com.sun.star.lang.XMultiServiceFactory]
}
itcl::body itcluno::Draw::getPageNames {} {
	return [$drawPages getElementNames]
}
itcl::body itcluno::Draw::insertPage {{position end} {layoutIndex -1}} {
	set pageList [getPageNames]
	if {[string equal $position end] || [llength $pageList] < $position} {
		set position [llength $pageList]
	}
#	if {$position < -1} {
#		set position -1
#	}
	$drawPages insertNewByIndex $position
	set page [getPageByIndex $position]
	if {[string length $page] == 0} {
		return -1
	}
	if {$layoutIndex != -1} {
		$page setPropertyValue Layout $layoutIndex
	}
	return $position
}
itcl::body itcluno::Draw::renamePage {oldname newname} {
	if {![checkPageName $oldname]} {
		return ""
	}
	if {[checkPageName $newname]} {
		return ""
	}
	set page [$drawPages getByName $oldname]
	$page setName $newname
	foreach oldIndex [array names _textHandleArray $oldname@@@*] {
		regsub $oldname $oldIndex $newname newIndex
		set _textHandleArray($newIndex) $_textHandleArray($oldIndex)
	}
	array unset _textHandleArray $oldname@@@*
	return $newname
}
itcl::body itcluno::Draw::deletePage {pageName} {
	if {![checkPageName $pageName]} {
		return ""
	}
	array unset _textHandleArray $pageName@@@*
	$drawPages remove $pageName
	return $pageName
}
itcl::body itcluno::Draw::activatePage {pageName} {
	if {![checkPageName $pageName]} {
		return ""
	}
	_getController
	set page [$drawPages getByName $pageName]
	$controller setCurrentPage $page
	return $pageName
}
itcl::body itcluno::Draw::setMasterText {rectangle text {type Text}} {
	set page [$masterPages getByIndex 0]
	set text [tcluno::simpress::setText $document $page $rectangle $text com.sun.star.drawing.${type}Shape]
	set _presentationObject($objectPrefix[incr _objectId]) $text
	return $objectPrefix$_objectId
}
itcl::body itcluno::Draw::setText {pageName rectangle text {type Text}} {
	if {![checkPageName $pageName]} {
		return ""
	}
	set page [$drawPages getByName $pageName]
	set text [tcluno::simpress::setText $document $page $rectangle $text com.sun.star.drawing.${type}Shape]
	set _presentationObject($objectPrefix[incr _objectId]) $text
	return $objectPrefix$_objectId
}
itcl::body itcluno::Draw::setPreparedText {pageName index string {propertyList ""}} {
	set page [$drawPages getByName $pageName]
	set text [$page getByIndex $index]
	set _textHandleArray($pageName@@@$index) $text
	foreach {name value} $propertyList {
		[$text getEnd] setPropertyValue $name $value
	}
	$text insertString [$text getText] $string 1
	return [$text getEnd]
}
itcl::body itcluno::Draw::appendPreparedHyperLink {pageName index string {propertyList ""}} {
	set textEnd [appendPreparedText $pageName $index $string $propertyList]
	set cursor [[$_textHandleArray($pageName@@@$index) getText] createTextCursor]
	$cursor setPropertyValue HyperLinkURL http://sourceforge.net
}
itcl::body itcluno::Draw::appendPreparedText {pageName index string {propertyList ""}} {
	if {![checkPageName $pageName]} {
		return ""
	}
	if {![info exists _textHandleArray($pageName@@@$index)]} {
		set page [$drawPages getByName $pageName]
		set text [$page getByIndex $index]
		set _textHandleArray($pageName@@@$index) $text
	}
	set textEnd [$_textHandleArray($pageName@@@$index) getEnd]
	foreach {name value} $propertyList {
		$textEnd setPropertyValue $name $value
	}
	if {[string length $string] > 0} {
		$textEnd setString $string
	}
	return $textEnd
}
itcl::body itcluno::Draw::appendPreparedTextParagraph {pageName index string {propertyList ""}} {
	set textEnd [appendPreparedText $pageName $index $string $propertyList]
	if {[string length $textEnd] == 0} {
		return ""
	}
	set textEnd [$_textHandleArray($pageName@@@$index) getEnd]
	$_textHandleArray($pageName@@@$index) insertControlCharacter $textEnd 0 0
	return $textEnd
}
itcl::body itcluno::Draw::setImage {pageName rectangle graphicURL} {
	if {![checkPageName $pageName]} {
		return ""
	}
	set page [$drawPages getByName $pageName]
	return [_setImage $page $rectangle $graphicURL]
}
itcl::body itcluno::Draw::setMasterImage {rectangle graphicURL} {
	set page [$masterPages getByIndex 0]
	return [_setImage $page $rectangle $graphicURL]
}
itcl::body itcluno::Draw::_setImage {pageHandle rectangle graphicURL} {
	foreach {x y w h unit} $rectangle break
	set position [tcluno_soffice::createPoint [list $x $y $unit]]
	set size [tcluno_soffice::createSize [list $w $h $unit]]
	set bitmaps [$document createInstance com.sun.star.drawing.BitmapTable]
	$bitmaps insertByName $bitmapPrefix[incr _bitmapId] $graphicURL
	set newURL [$bitmaps getByName $bitmapPrefix$_bitmapId]
	set tmpImage [$document createInstance com.sun.star.drawing.GraphicObjectShape]
	OpenOffice::setPropertyValues $tmpImage [list GraphicURL $newURL]
	$tmpImage setPosition $position
	$tmpImage setSize $size
#	array set properties [getPropertyValues $tmpImage ""]
#	unset -nocomplain properties(Preview)
#	unset -nocomplain properties(MetaFile)
#	parray properties
	$pageHandle add $tmpImage
	set _presentationObject($objectPrefix[incr _objectId]) $tmpImage
	return $objectPrefix$_objectId
}
itcl::body itcluno::Draw::setPreparedImage {pageName index graphicURL} {
	if {![checkPageName $pageName]} {
		return ""
	}
	set page [$drawPages getByName $pageName]
	set text [$page getByIndex $index]
	set position [lindex [$text getPosition] 1]
	foreach e $position {
		set pos([lindex $e 0]) [lindex $e 1]
	}
	set size [lindex [$text getSize] 1]
	foreach e $size {
		set pos([lindex $e 0]) [lindex $e 1]
	}
	set bitmaps [$document createInstance com.sun.star.drawing.BitmapTable]
	$bitmaps insertByName $bitmapPrefix[incr _bitmapId] $graphicURL
	set newURL [$bitmaps getByName $bitmapPrefix$_bitmapId]
	set tmpImage [$document createInstance com.sun.star.drawing.GraphicObjectShape]
	OpenOffice::setPropertyValues $tmpImage [list GraphicURL $newURL]
	$tmpImage setPosition [tcluno_soffice::createPoint [list [expr {$pos(X)}] [expr {$pos(Y)}]]]
	$tmpImage setSize [tcluno_soffice::createSize [list [expr {$pos(Width)}] [expr {$pos(Height)}]]]
	$page add $tmpImage
	$page remove $text
	return $tmpImage
}
itcl::body itcluno::Draw::getDocumentProperties {{propertyList ""}} {
	return [getPropertyValues $document $propertyList]
}
itcl::body itcluno::Draw::getMasterPageProperties {{propertyList ""}} {
	set page [$masterPages getByIndex 0]
	return [getPropertyValues $page $propertyList]
}
itcl::body itcluno::Draw::getPageProperties {pageName {propertyList ""}} {
	if {![checkPageName $pageName]} {
		return ""
	}
	set page [$drawPages getByName $pageName]
	return [getPropertyValues $page $propertyList]
}
itcl::body itcluno::Draw::getPreparedObjectProperties {pageName index {propertyList ""}} {
	if {![checkPageName $pageName]} {
		return ""
	}
	set page [$drawPages getByName $pageName]
	set object [$page getByIndex $index]
	return [getPropertyValues $object $propertyList]
}
itcl::body itcluno::Draw::getObjectProperties {objectId {propertyList ""}} {
	set object $_presentationObject($objectId)
	return [getPropertyValues $object $propertyList]
}
itcl::body itcluno::Draw::setPagePropertyValues {pageName propertyList} {
	if {![checkPageName $pageName]} {
		return ""
	}
	set page [$drawPages getByName $pageName]
	OpenOffice::setPropertyValues $page $propertyList
}
itcl::body itcluno::Draw::setPropertyValues {objectId propertyList} {
	OpenOffice::setPropertyValues $_presentationObject($objectId) $propertyList
}
itcl::body itcluno::Draw::setPreparedObjectProperties {pageName index propertyList} {
	if {![checkPageName $pageName]} {
		return ""
	}
	set page [$drawPages getByName $pageName]
	set object [$page getByIndex $index]
	OpenOffice::setPropertyValues $object $propertyList
}
itcl::body itcluno::Draw::createTable {pageName rectangle} {
	set model [createOLE2Object $pageName $rectangle table]
	if {[string length $model] == 0} {
		return ""
	}
	return [itcluno::SpreadSheet $this#auto -handle $model]
}
itcl::body itcluno::Draw::createChart {pageName rectangle} {
	set model [createOLE2Object $pageName $rectangle chart]
	return [itcluno::ChartDocument $this#auto -handle $model]
}
itcl::body itcluno::Draw::createShape {pageName rectangle shapeType {propertyList ""}} {
	if {![checkPageName $pageName]} {
		return ""
	}
	if {![info exists _pageHandles($pageName)]} {
		set _pageHandles($pageName) [$drawPages getByName $pageName]
	}
	set provider [$document getScriptProvider]
	set pageIdx [lsearch -exact [getPageNames] $pageName]
	if {[llength $rectangle] > 5} {
		set script [$provider getScript "vnd.sun.star.script:Standard.itcluno.DrawPolyLine?language=Basic&location=application"]
		if {[expr {[llength $rectangle]%2}] == 1} {
			#	we have a unit
			set unit [tcluno_soffice::getUnitFactor [lindex $rectangle end]]
			set rectangle [lrange $rectangle 0 end-1]
			set tmp [list]
			foreach e $rectangle {
				lappend tmp [expr {int($e*$unit)}]
			}
			set rectangle [join $tmp " "]
			set shape [$script invoke $shapeType $rectangle $pageIdx]
		}
	} else {
		set script [$provider getScript "vnd.sun.star.script:Standard.itcluno.DrawShape?language=Basic&location=application"]
		foreach {x y w h unit} $rectangle break
		set x [expr {int($x*[::tcluno_soffice::getUnitFactor $unit])}]
		set y [expr {int($y*[::tcluno_soffice::getUnitFactor $unit])}]
		set w [expr {int($w*[::tcluno_soffice::getUnitFactor $unit])}]
		set h [expr {int($h*[::tcluno_soffice::getUnitFactor $unit])}]
		set shape [$script invoke $shapeType $x $y $w $h $pageIdx]
	}
	OpenOffice::setPropertyValues $shape $propertyList
	return $shape
}
itcl::body itcluno::Draw::group {pageName shapeList name} {
	if {![checkPageName $pageName]} {
		return ""
	}
	if {![info exists _pageHandles($pageName)]} {
		set _pageHandles($pageName) [$drawPages getByName $pageName]
	}
	set page [$drawPages getByName $pageName]
	set xGroup [createShape $pageName [list 0 0 0 0] Group]
	set xShapes [itcluno::OfficeUtilities::queryInterface $page com.sun.star.drawing.XShapes]
	$xShapes add $xGroup
	set xShapesGroup [itcluno::OfficeUtilities::queryInterface $xGroup com.sun.star.drawing.XShapes]
	foreach shape $shapeList {
		$xShapesGroup add $shape
	}
	return $xGroup
}
if 0 {
itcl::body itcluno::Draw::createShape {pageName rectangle shapeType {propertyList ""}} {
	if {![checkPageName $pageName]} {
		return ""
	}
	if {![info exists _pageHandles($pageName)]} {
		set _pageHandles($pageName) [$drawPages getByName $pageName]
	}
	set shape [$document createInstance com.sun.star.drawing.${shapeType}Shape]
	foreach {x y w h unit} $rectangle break
#	set xShape [itcluno::OfficeUtilities::queryInterface $shape com.sun.star.drawing.XShape]
	set size [::tcluno_soffice::createSize [list $w $h $unit]]
	$shape setSize $size
	if {[string length $_point] == 0} {
		set _point [::tcluno_soffice::createPoint [list $x $y $unit]]
	}
	$shape setPosition $_point
	OpenOffice::setPropertyValues $shape $propertyList
#	puts stderr "$_pageHandles($pageName) add $shape"
	if {![info exists _shapes($pageName)]} {
		set _shapes($pageName) [itcluno::OfficeUtilities::queryInterface $_pageHandles($pageName) com.sun.star.drawing.XShapes]
	}
	if {[catch {$_shapes($pageName) add $shape} msg]} {
		puts stderr "msg:$msg:"
		puts stderr "******************** get new page handle for $pageName********************"
#		set _pageHandles($pageName) [$drawPages getByName $pageName]
#		$_pageHandles($pageName) add $shape
	}
	return $shape
}
itcl::body itcluno::Draw::group {pageName shapeList name} {
#	return ""
	if {![checkPageName $pageName]} {
		return ""
	}
	if {![info exists _pageHandles($pageName)]} {
		set _pageHandles($pageName) [$drawPages getByName $pageName]
	}
#	set page [$drawPages getByName $pageName]
	set xGroup [createShape $pageName [list 0 0 20000 30000] Group]
	set xShapes [itcluno::OfficeUtilities::queryInterface $_pageHandles($pageName) com.sun.star.drawing.XShapes]
	if {[catch {$xShapes add $xGroup}]} {
		puts stderr "******************** get new page handle (in group method) ********************"
		set _pageHandles($pageName) [$drawPages getByName $pageName]
		set xShapes [itcluno::OfficeUtilities::queryInterface $_pageHandles($pageName) com.sun.star.drawing.XShapes]
		$xShapes add $xGroup
	}
	set xShapesGroup [itcluno::OfficeUtilities::queryInterface $xGroup com.sun.star.drawing.XShapes]
	foreach shape $shapeList {
		$xShapesGroup add $shape
	}
	return $xShapesGroup
}
}; #if 0
itcl::body itcluno::Draw::exportToPdf {filename} {
	itcluno::OpenOffice::exportToPdf $filename impress_pdf_Export
}
itcl::body itcluno::Draw::checkPageName {pageName} {
	set pageList [getPageNames]
	if {[lsearch -exact $pageList $pageName] < 0} {
		return 0
	}
	return 1
}
itcl::body itcluno::Draw::getPageByName {pageName} {
	if {![checkPageName $pageName]} {
		return ""
	}
	return [$drawPages getByName $pageName]
}
itcl::body itcluno::Draw::getPageByIndex {index} {
	set nameList [getPageNames]
	if {$index < 0 || $index >= [llength $nameList]} {
		return ""
	}
	return [$drawPages getByIndex $index]
}
itcl::body itcluno::Draw::checkForFilter {filename filterSequence} {
	if {[string equal $filename $factory]} {
		return
	}
	set extension [file extension $filename]
	switch -- $extension {
		".ppt" {
			set filterProperty [$desktop tcluno::createUnoStructHelper com.sun.star.beans.PropertyValue {FilterName -1 {MS PowerPoint 97} 0}]
			$desktop tcluno::appendUnoSequence $filterSequence $filterProperty
			return
		}
		".odg" - ".otg" {
			return
		}
	}
	return -code error -errorinfo "unsupported extension [file extension $filename]"
}
itcl::body itcluno::Draw::createOLE2Object {pageName rectangle type} {
	if {![checkPageName $pageName]} {
		return ""
	}
	set page [$drawPages getByName $pageName]
#	set msf [itcluno::OfficeUtilities::queryInterface $document com.sun.star.lang.XMultiServiceFactory]
	set ole2 [$multiServiceFactory createInstance com.sun.star.drawing.OLE2Shape]
	set shape [itcluno::OfficeUtilities::queryInterface $ole2 com.sun.star.drawing.XShape]
	$page add $shape
	foreach {x y w h unit} $rectangle break
	set point [::tcluno_soffice::createPoint [list $x $y $unit]]
	$shape setPosition $point
	set size [::tcluno_soffice::createSize [list $w $h $unit]]
	$shape setSize $size
	set shapeProp [itcluno::OfficeUtilities::queryInterface $shape com.sun.star.beans.XPropertySet]
	$shapeProp setPropertyValue CLSID $clsid($type)
	return [$shapeProp getPropertyValue Model]
}
itcl::body itcluno::Draw::_getController {} {
	if {[string length $controller] > 0} {
		return $controller
	}
	set controller [$document getCurrentController]
	return $controller
}
itcl::body itcluno::Draw::setFilterList {} {
	array set filterList [list \
			.html draw_html_Export \
			.htm draw_html_Export \
			.pdf writer_pdf_Export \
			.swf draw_flash_Export \
			.xhtml "XHTML Draw File" \
			.bmp draw_bmp_Export \
			.emf draw_emf_Export \
			.eps draw_eps_Export \
			.gif draw_gif_Export \
			.jpg draw_jpg_Export \
			.jpeg draw_jpg_Export \
			.jfif draw_jpg_Export \
			.jif draw_jpg_Export \
			.jpe draw_jpg_Export \
			.met draw_met_Export \
			.pbm draw_pbm_Export \
			.pct draw_pct_Export \
			.pgm draw_pgm_Export \
			.png draw_png_Export \
			.ppm draw_ppm_Export \
			.ras draw_ras_Export \
			.svg draw_svg_Export \
			.svm draw_svm_Export \
			.tif draw_tif_Export \
			.tiff draw_tif_Export \
			.wmf draw_wmf_Export \
			.xpm draw_xpm_Export \
		]
}
