#*************************************************************************
#*
#*  $RCSfile: Writer.tcl,v $
#*
#*  $Revision: 224 $
#*
#*  last change: $Author: grosserw $ $Date: 2007-03-13 10:10:00 +0100 (Di, 13 Mrz 2007) $
#*
#*  The Contents of this file are made available subject to the terms of
#*  the following license
#*
#*  - BSD License
#*
#*  see file license.terms in this directory
#*
#*  Copyright 2005-2006 Wolfgang Grosser
#*
#*  Author: Wolfgang Grosser
#*
#*   All Rights Reserved.
#*
#*   Contributor(s): 
#*
#*
#************************************************************************/

package require Itcl

itcl::class itcluno::Writer {
	inherit OpenOffice

	protected variable textHandle ""
	protected variable _bitmapId 0
	protected variable bitmapPrefix _writer

	private variable _backgroundImage ""
	private variable _maximumNumberingDepth 10

	constructor {args} {eval OpenOffice::constructor private:factory/swriter $args} {}
	public method appendText {string {propertyList ""}}
	public method appendParagraph {string {propertyList ""}}
	public method appendTable {rows columns}
	public method appendFormula {formula {propertyList ""}}
	public method appendTextTable {table}
	public method getStyleFamilies {}
	public method getStyles {family}
	public method getStyleProperties {family style {propertyList ""}}
	public method getPropertyValues {object {propertyList ""}}
	public method getNumberingStyleProperties {styleName {idxList ""}}

	public method setNumberingStyleProperties {styleName idxList propertyList}
	public method setDocumentProperties {propertyList}
	public method setStyleProperties {family style propertyList}

	public method exportToPdf {filename}

	protected method getStyleDisplayNames {family}
	protected method checkForFilter {filename filterSequence}

	private method setFilterList {}
}

itcl::body itcluno::Writer::constructor {args} {
	setFilterList
	set bitmapPrefix "_${this}_bitmap_@@@_"
	if {[string length $handle] != 0} {
		set document $handle
	} else {
		if {[string length $filename] == 0} {
			set filename $factory
		}
		if {[catch {checkForFilter $filename $filterSequence} msg]} {
			puts stderr $msg
			#	try opening but without guarantee what happens
			openFile $filename $filterSequence
			return
		}
		openFile $filename $filterSequence
	}
	set textHandle [$document getText]
}
itcl::body itcluno::Writer::appendText {string {propertyList ""}} {
	set textEnd [$textHandle getEnd]
	foreach {name value} $propertyList {
		$textEnd setPropertyValue $name $value
	}
	if {[string length $string] > 0} {
		$textEnd setString $string
	}
	return $textEnd
}
itcl::body itcluno::Writer::appendParagraph {string {propertyList ""}} {
	set textEnd [appendText $string $propertyList]
	set tmp $textEnd
	set textEnd [$textHandle getEnd]
	$textHandle insertControlCharacter $textEnd 0 0
	return $tmp
}
itcl::body itcluno::Writer::appendTable {rows columns} {
	set tableInstance [$document createInstance com.sun.star.text.TextTable]
	set table [itcluno::OfficeUtilities::queryInterface $tableInstance com.sun.star.text.XTextTable]
	$table initialize $rows $columns
	set textEnd [$textHandle getEnd]
	set text [$textEnd getText]
	$text insertTextContent $textEnd $table 0

	return [itcluno::TextTable $this#auto -handle $table]
}
itcl::body itcluno::Writer::appendFormula {formula {propertyList ""}} {
	set object [$document createInstance com.sun.star.text.TextEmbeddedObject]
	$object setPropertyValue CLSID $clsid(formula)
	$object setPropertyValue AnchorType [itcluno::OfficeUtilities::enum com.sun.star.text.TextContentAnchorType AS_CHARACTER]
	set textEnd [$textHandle getEnd]
	set text [$textEnd getText]

	#	Insert the new object (afterwards you have access to it),
	#	so don't change the order of inserting and retrieving object
	$text insertTextContent $textEnd $object 0

	#	Now you can retrieve the embedded object
	set embedded [$object getEmbeddedObject]
	$embedded setPropertyValue Formula $formula
	setPropertyValues $embedded $propertyList
	return $embedded
}
itcl::body itcluno::Writer::appendTextTable {table} {
	set textEnd [$textHandle getEnd]
	set text [$textEnd getText]
	$text insertTextContent $textEnd [$table document] 0
}
itcl::body itcluno::Writer::getStyleFamilies {} {
	set styleFamilies [$document getStyleFamilies]
	return [$styleFamilies getElementNames]
}
itcl::body itcluno::Writer::getStyles {family} {
	set styleFamily [$document getStyleFamilies]
	set family [$styleFamily getByName $family]
	return [$family getElementNames]
}
itcl::body itcluno::Writer::getStyleProperties {family style {propertyList ""}} {
	set styleFamily [$document getStyleFamilies]
	set family [$styleFamily getByName $family]
	if {[string equal $family {TCL_NONE}]} {
		return [list]
	}
	set style [$family getByName $style]
	if {[string equal $style {TCL_NONE}]} {
		return [list]
	}
	return [getPropertyValues $style $propertyList]
}
itcl::body itcluno::Writer::getPropertyValues {object {propertyList ""}} {
	return [::itcluno::OpenOffice::getPropertyValues $object $propertyList]
}
itcl::body itcluno::Writer::setNumberingStyleProperties {styleName idxList propertyList} {
	if {[string length $styleName] > 0} {
		array set tmp [getStyleProperties NumberingStyles $styleName]
		set numRules $tmp(NumberingRules)
	} else {
		set numRules [$document createInstance "com.sun.star.text.NumberingRules"]
	}
	set xNum [itcluno::OfficeUtilities::queryInterface $numRules com.sun.star.container.XIndexAccess]
	set xReplacer [itcluno::OfficeUtilities::queryInterface $xNum com.sun.star.container.XIndexReplace]
	if {[string length $idxList] == 0} {
		set idxList [list]
		for {set idx 0} {$idx < $_maximumNumberingDepth} {incr idx} {
			lappend idxList $idx
		}
	}
	foreach idx $idxList {
		set props($idx) [$xNum getByIndex $idx]
		set propList [list]
		foreach e $props($idx) {
			foreach {struct values} $e break
			set values [join $values " "]
			set searchIdx [lsearch -exact $propertyList [lindex $values 1]]
			if {$searchIdx < 0 || [expr {$searchIdx%2}] == 1} {
				if {[string equal [lindex $values 1] {BulletFont}]} {
					set values [join [join [lindex [lindex $values 5] 0] " "] " "]
					set valueList [list]
					for {set i 2} {$i < [llength $values]} {incr i 2} {
						set tmpValue [lindex $values $i]
						if {[string equal $tmpValue {NONE}]} {
							set tmpValue 0
						}
						lappend valueList $tmpValue
					}
					set value [$desktop tcluno::createUnoStructHelper com.sun.star.awt.FontDescriptor [list StarSymbol 28 0 "" 0 20 0 0.0 0.0 0 0 0 0.0 False False 5]]
					lappend propList [list BulletFont -1 $value 0]
				} else {
					lappend propList [list [lindex $values 1] -1 [lindex $values 5] 0]
				}
				continue
			}
			set tmpVal [list]
			lappend tmpVal [lindex $values 1]
			lappend tmpVal -1
			if {[string equal [lindex $values 1] {BulletFont}]} {
				set value [$desktop tcluno::createUnoStructHelper com.sun.star.awt.FontDescriptor [list Webdings 28 0 "" 0 20 0 0.0 0.0 0 0 0 0.0 False False 5]]
				lappend tmpVal $value
			} else {
				lappend tmpVal [lindex $propertyList [expr {$searchIdx+1}]]
			}
			lappend tmpVal 0
#			puts stderr $tmpVal
			lappend propList $tmpVal
		}
		set suffProp [itcluno::OfficeUtilities::makeSequence $propList com.sun.star.beans.PropertyValue]
		$xReplacer replaceByIndex $idx $suffProp
	}
	setStyleProperties NumberingStyles $styleName [list NumberingRules $xReplacer]
}
itcl::body itcluno::Writer::getNumberingStyleProperties {styleName {idxList ""}} {
	array set tmp [getStyleProperties NumberingStyles $styleName]
	set numRules $tmp(NumberingRules)
	set xNum [itcluno::OfficeUtilities::queryInterface $numRules com.sun.star.container.XIndexAccess]
	set xReplacer [itcluno::OfficeUtilities::queryInterface $xNum com.sun.star.container.XIndexReplace]
	if {[string length $idxList] == 0} {
		set idxList [list]
		for {set idx 0} {$idx < $_maximumNumberingDepth} {incr idx} {
			lappend idxList $idx
		}
	}
	set resultList [list]
	foreach idx $idxList {
		set props($idx) [join [join [join [$xNum getByIndex $idx] " "] " "] " "]
		set tmpList [list]
		set len [llength $props($idx)]
		for {set i 2} {$i < $len} {incr i 9} {
			lappend tmpList [lindex $props($idx) $i] [lindex $props($idx) [expr {$i+4}]]
		}
		lappend resultList $idx $tmpList
	}
	return $resultList
}
itcl::body itcluno::Writer::setDocumentProperties {propertyList} {
	setPropertyValues $document $propertyList
}
itcl::body itcluno::Writer::setStyleProperties {family style propertyList} {
	set idx [lsearch -exact $propertyList {BackGraphicURL}]
	incr idx
#	if {$idx >= 0} {
#		set bitmaps [$document createInstance com.sun.star.drawing.BitmapTable]
#		incr idx
#		set _bitmapId [incr _bitmapId]
#		puts stderr "$bitmaps insertByName $bitmapPrefix$_bitmapId [lindex $propertyList $idx]"
#		puts stderr [join [tcluno_soffice::show_method_infos $bitmaps 1] "\n"]
#		puts stderr [$bitmaps hasElements]
#		$bitmaps insertByName $bitmapPrefix$_bitmapId [itcluno::OfficeUtilities::makeSequence [lindex $propertyList $idx]]
#		set newURL [$bitmaps getByName $bitmapPrefix$_bitmapId]
#		set tmpImage [$document createInstance com.sun.star.drawing.GraphicObjectShape]
#		OpenOffice::setPropertyValues $tmpImage [list GraphicURL $newURL]
#		set propertyList [lreplace $propertyList $idx $idx $newURL]
#	}
	set styleFamilies [$document getStyleFamilies]
	set styleFamily [$styleFamilies getByName $family]
	if {[string equal $styleFamily {TCL_NONE}]} {
		return
	}
	set style [$styleFamily getByName $style]
	if {[string equal $style {TCL_NONE}]} {
		return
	}
	setPropertyValues $style $propertyList
}
itcl::body itcluno::Writer::exportToPdf {filename} {
	itcluno::OpenOffice::exportToPdf $filename writer_pdf_Export
}
itcl::body itcluno::Writer::getStyleDisplayNames {family} {
	set styleFamily [$document getStyleFamilies]
	set family [$styleFamily getByName $family]
	set counter [$family getCount]
	set result [list]
	for {set i 0} {$i < $counter} {
		lappend result [[$family getByIndex $i] getPropertyValue LinkDisplayName]
	}
	return $result
}
itcl::body itcluno::Writer::checkForFilter {filename filterSequence} {
	if {[string equal $filename $factory]} {
		return
	}
	switch -- [file extension $filename] {
		".doc" {
			set filterProperty [$desktop tcluno::createUnoStructHelper com.sun.star.beans.PropertyValue {FilterName -1 {MS Word 97} 0}]
			$desktop tcluno::appendUnoSequence $filterSequence $filterProperty
			return
		}
		".rtf" {
			set filterProperty [$desktop tcluno::createUnoStructHelper com.sun.star.beans.PropertyValue [list FilterName -1 "Rich Text Format" 0]]
			$desktop tcluno::appendUnoSequence $filterSequence $filterProperty
			return
		}
		".stw" - ".sxw" - ".ott" - ".odt" - ".txt" {
			return
		}
	}
	return -code error -errorinfo "unsupported extension [file extension $filename]"
}
itcl::body itcluno::Writer::setFilterList {} {
	array set filterList [list \
			.pdf writer_pdf_Export \
			.bib BibTeX_Writer \
			.tex LaTeX_Writer \
			.xhtml "XHTML Writer File" \
		]
}
