#*************************************************************************
#*
#*  $RCSfile: Writer.tcl,v $
#*
#*  $Revision:188 $
#*
#*  last change: $Author: grosserw $ $Date: 2006-01-30 08:51:55 +0100 (Mo, 30 Jan 2006) $
#*
#*  The Contents of this file are made available subject to the terms of
#*  the following license
#*
#*  - BSD License
#*
#*  see file license.terms in this directory
#*
#*  Copyright 2005-2006 Wolfgang Grosser
#*
#*  Author: Wolfgang Grosser
#*
#*   All Rights Reserved.
#*
#*   Contributor(s): 
#*
#*
#************************************************************************/


package require tclurtp
package require tcluno
package require itcluno 0.2


set name "Writer:simple"
set description($name) {
write a simple "hallo" to the Writer document
}; # END OF EXAMPLE-DESCRIPTION
set example($name) {
package require itcluno 0.2

catch {itcl::delete object new_document}

itcluno::Writer new_document
new_document appendText hallo

catch {tcluno_soffice::windowTitle [new_document cget -desktop] "TCLUNO - DEMO: simple"}
}; # END OF EXAMPLE-CODE

set name "Writer:ParagraphStyles"
set description($name) {
This examples shows all properties of all styles of all style families.\nIt produces about 350 pages in the writer document
}; # END OF EXAMPLE-DESCRIPTION
set example($name) {
package require itcluno 0.2

catch {itcl::delete object new_document}

itcluno::Writer new_document
set numberingStyleName "Numbering 5"
set paraStyle "Text body"
new_document setNumberingStyleProperties $numberingStyleName \
	[list 0 1 2] \
	[list Prefix "(" Suffix ")."]
set numberingStyle [list NumberingStyleName "Numbering 1" NumberingLevel %d]
set tmpFormat [concat [list ParaStyleName "Heading 1"] [format $numberingStyle 0]]
new_document appendParagraph "StyleFamilies" $tmpFormat
set familyCount 0
foreach family [lsort [new_document getStyleFamilies]] {
	if {$familyCount == 0} {
		set familyStartValue "NumberingStartValue 1"
		incr familyCount
	}
	set tmpFormat [concat [list ParaStyleName "Heading 2"] [format $numberingStyle 1]]
	new_document appendParagraph "$family" $tmpFormat
	set styleCount 0
	set counter 0
	foreach style [lsort [new_document getStyles $family]] {
		if {$styleCount == 0} {
			set styleStartValue "NumberingStartValue 1"
			incr styleCount
		}
		set tmpFormat [concat [list ParaStyleName "Heading 3"] [format $numberingStyle 2]]
		new_document appendParagraph "$style" $tmpFormat
		set props [new_document getStyleProperties $family $style]
		set propList [list]
		foreach {prop value} $props {
			lappend propList [list $prop\t$value]
		}
		new_document appendParagraph [join $propList "\n"] [list ParaStyleName "Text body" NumberingStyleName ""]
		incr counter
		if {$counter > 1000} break
	}
}

catch {tcluno_soffice::windowTitle [new_document cget -desktop] "TCLUNO - DEMO: ParagraphStyles"}
}; # END OF EXAMPLE-CODE

set name "Writer:Enumeration with Graphics"
set description($name) {
This example shows the use of the enumeration capabilities of a Writer document
}; # END OF EXAMPLE-DESCRIPTION
set example($name) {
package require itcluno 0.2

catch {itcl::delete object new_document}

itcluno::Writer new_document

set numberingStyleName "List 1"
array set properties [new_document getNumberingStyleProperties $numberingStyleName]
parray properties
set coffeeSize [::tcluno_soffice::createSize [list 500 500]]
set arrowSize [::tcluno_soffice::createSize [list 370 190]]
array set properties [new_document getNumberingStyleProperties $numberingStyleName]
set lineBulletDir [itcluno::OfficeUtilities::filenameToURL $OOoTopDir/share/gallery/bullets]
new_document setNumberingStyleProperties $numberingStyleName \
	[list 0 2] \
	[list \
		NumberingType 8 \
		GraphicSize $arrowSize \
		GraphicURL [file join $lineBulletDir coffee_5.gif] \
		BulletFontName Webdings \
		BulletId 227 \
		BulletFont 37 \
	]
unset properties
array set properties [new_document getNumberingStyleProperties $numberingStyleName [list 0 2]]
parray properties
new_document setNumberingStyleProperties $numberingStyleName \
	[list 0] \
	[list \
		NumberingType 8 \
		GraphicSize $coffeeSize \
		GraphicURL [file join $lineBulletDir coffee_5.gif] \
	]
new_document setNumberingStyleProperties $numberingStyleName \
	[list 2] \
	[list \
		NumberingType 8 \
		GraphicSize $arrowSize \
		GraphicURL [file join $lineBulletDir bluarrow.gif] \
	]
set paraStyle "Text body"
set docHandle [new_document document]
set paraTabStops [itcluno::OfficeUtilities::makeSequence [list \
	[list 6000 0 [$docHandle ::tcluno::getTclunoChar [list CharClass "."]] [$docHandle ::tcluno::getTclunoChar [list CharClass "."]]] \
	[list 10000 3 [$docHandle ::tcluno::getTclunoChar [list CharClass ","]] [$docHandle ::tcluno::getTclunoChar [list CharClass "-"]]] \
	[list 15000 2 [$docHandle ::tcluno::getTclunoChar [list CharClass ","]] [$docHandle ::tcluno::getTclunoChar [list CharClass "-"]]] \
] com.sun.star.style.TabStop]
new_document setStyleProperties ParagraphStyles $paraStyle \
	[list ParaTabStops $paraTabStops]
set numberingStyle [list \
	ParaStyleName $paraStyle \
	NumberingStyleName $numberingStyleName \
	NumberingLevel %d\
]
new_document appendParagraph "\tThis is a numbering level 1" [eval list [format $numberingStyle 0]]
new_document appendParagraph "\tThis is level 2 of numbering\t3,1" [eval list [format $numberingStyle 1] NumberingStartValue 18]
new_document appendText "\tThis is level 2 of numbering" [eval list [format $numberingStyle 1]]
new_document appendParagraph ", but with bold addition" [list CharWeight [itcluno::OfficeUtilities::constant com.sun.star.awt.FontWeight BOLD]]
new_document appendText "\tThis is level 2 of numbering" [format $numberingStyle 1]
new_document appendParagraph ", but with red wavelike underlining" \
	[list \
		CharUnderlineHasColor True \
		CharUnderlineColor 0xff0000 \
		CharUnderline [itcluno::OfficeUtilities::constant com.sun.star.awt.FontUnderline WAVE] \
		CharWeight [itcluno::OfficeUtilities::constant com.sun.star.awt.FontWeight BOLD] \
	]
new_document appendParagraph "\tThis is level 2 of numbering" [format $numberingStyle 1]
new_document appendParagraph "\tThis is level 3 of numbering" [eval list [format $numberingStyle 2]  NumberingStartValue 1]
new_document appendParagraph "\tThis is level 3 of numbering" [format $numberingStyle 2]
new_document appendParagraph "\tThis is again level 1 of numbering" [format $numberingStyle 0]
new_document appendParagraph "\tThis is again level 1 of numbering" [format $numberingStyle 0]
new_document appendParagraph "\tdecimal alignment\t3,1\tright aligned" [format $numberingStyle 0]
new_document appendParagraph "\tdecimal alignment\t123,1\talso right aligned" [format $numberingStyle 0]

catch {tcluno_soffice::windowTitle [new_document cget -desktop] "TCLUNO - DEMO: Enumeration with Graphics"}
}; # END OF EXAMPLE-CODE

set name "Writer:Enumeration and Tabulators"
set description($name) {
This example shows the use of the tabulator and the enumeration capabilities of a Writer document
}; # END OF EXAMPLE-DESCRIPTION
set example($name) {
package require itcluno 0.2

catch {itcl::delete object new_document}

itcluno::Writer new_document

set numberingStyleName "List 1"
array set properties [new_document getNumberingStyleProperties $numberingStyleName]
parray properties
new_document setNumberingStyleProperties $numberingStyleName \
	[list 0 2] \
	[list \
		NumberingType 6 \
		BulletFontName Webdings \
		BulletId 227 \
		BulletFont 37 \
	]
unset properties
array set properties [new_document getNumberingStyleProperties $numberingStyleName [list 0 2]]
parray properties
new_document setNumberingStyleProperties $numberingStyleName \
	[list 0 2] \
	[list \
		NumberingType 6 \
		BulletFontName Webdings \
		BulletId 227 \
		BulletFont 37 \
	]
unset properties
array set properties [new_document getNumberingStyleProperties $numberingStyleName [list 0 2]]
parray properties
set paraStyle "Text body"
set docHandle [new_document document]
set paraTabStops [itcluno::OfficeUtilities::makeSequence [list \
	[list 6000 0 [$docHandle ::tcluno::getTclunoChar [list CharClass "."]] [$docHandle ::tcluno::getTclunoChar [list CharClass "."]]] \
	[list 10000 3 [$docHandle ::tcluno::getTclunoChar [list CharClass ","]] [$docHandle ::tcluno::getTclunoChar [list CharClass "-"]]] \
	[list 15000 2 [$docHandle ::tcluno::getTclunoChar [list CharClass ","]] [$docHandle ::tcluno::getTclunoChar [list CharClass "-"]]] \
] com.sun.star.style.TabStop]
new_document setStyleProperties ParagraphStyles $paraStyle [list ParaTabStops $paraTabStops]
set numberingStyle [list ParaStyleName $paraStyle NumberingStyleName $numberingStyleName NumberingLevel %d]
new_document appendParagraph "\tThis is a numbering level 1" [eval list [format $numberingStyle 0]]
new_document appendParagraph "\tThis is level 2 of numbering\t3,1" [eval list [format $numberingStyle 1] NumberingStartValue 18]
new_document appendText "\tThis is level 2 of numbering" [eval list [format $numberingStyle 1]]
new_document appendParagraph ", but with bold addition" [list CharWeight [itcluno::OfficeUtilities::constant com.sun.star.awt.FontWeight BOLD]]
new_document appendText "\tThis is level 2 of numbering" [format $numberingStyle 1]
new_document appendParagraph ", but with red wavelike underlining" \
	[list \
		CharUnderlineHasColor True \
		CharUnderlineColor 0xff0000 \
		CharUnderline [itcluno::OfficeUtilities::constant com.sun.star.awt.FontUnderline WAVE] \
		CharWeight [itcluno::OfficeUtilities::constant com.sun.star.awt.FontWeight BOLD] \
	]
new_document appendParagraph "\tThis is level 2 of numbering" [format $numberingStyle 1]
new_document appendParagraph "\tThis is level 3 of numbering" [eval list [format $numberingStyle 2]  NumberingStartValue 1]
new_document appendParagraph "\tThis is level 3 of numbering" [format $numberingStyle 2]
new_document appendParagraph "\tThis is again level 1 of numbering" [format $numberingStyle 0]
new_document appendParagraph "\tThis is again level 1 of numbering" [format $numberingStyle 0]
new_document appendParagraph "\tdecimal alignment\t3,1\tright aligned" [format $numberingStyle 0]
new_document appendParagraph "\tdecimal alignment\t123,1\talso right aligned" [format $numberingStyle 0]

catch {tcluno_soffice::windowTitle [new_document cget -desktop] "TCLUNO - DEMO: Enumeration and Tabulators"}
}; # END OF EXAMPLE-CODE

set name "Writer:Simple Numbering"
set description($name) {
This example shows the numbering capabilities of a writer document
}; # END OF EXAMPLE-DESCRIPTION
set example($name) {
package require itcluno 0.2

catch {itcl::delete object new_document}

itcluno::Writer new_document

set numberingStyleName "Numbering 1"
set paraStyle "Text body"
new_document setStyleProperties ParagraphStyles $paraStyle [list NumberingStyleName "Numbering 1"]
set docHandle [new_document document]
set paraTabStops [itcluno::OfficeUtilities::makeSequence [list \
	[list 6000 0 [$docHandle ::tcluno::getTclunoChar [list CharClass "."]] [$docHandle ::tcluno::getTclunoChar [list CharClass "."]]] \
	[list 10000 3 [$docHandle ::tcluno::getTclunoChar [list CharClass ","]] [$docHandle ::tcluno::getTclunoChar [list CharClass "-"]]] \
	[list 15000 2 [$docHandle ::tcluno::getTclunoChar [list CharClass ","]] [$docHandle ::tcluno::getTclunoChar [list CharClass "-"]]] \
] com.sun.star.style.TabStop]
new_document setStyleProperties ParagraphStyles $paraStyle [list ParaTabStops $paraTabStops]
set numberingStyle [list ParaStyleName $paraStyle NumberingStyleName "Numbering 1" NumberingLevel %d]
new_document appendParagraph "\tThis is a numbering level 1" [eval list [format $numberingStyle 0]]
new_document appendParagraph "\tThis is level 2 of numbering\t3,1" [eval list [format $numberingStyle 1]]
new_document appendText "\tThis is level 2 of numbering" [eval list [format $numberingStyle 1]]
new_document appendParagraph ", but with bold addition" \
	[list CharWeight [itcluno::OfficeUtilities::constant com.sun.star.awt.FontWeight BOLD]]
new_document appendText "\tThis is level 2 of numbering"
new_document appendParagraph ", but with red wavelike underlining" \
	[list \
		CharUnderlineHasColor True \
		CharUnderlineColor 0xff0000 \
		CharUnderline [itcluno::OfficeUtilities::constant com.sun.star.awt.FontUnderline WAVE] \
		CharWeight [itcluno::OfficeUtilities::constant com.sun.star.awt.FontWeight BOLD] \
	]
new_document appendParagraph "\tThis is level 2 of numbering" [format $numberingStyle 2]
new_document appendParagraph "\tThis is level 3 of numbering"
new_document appendParagraph "\tThis is level 3 of numbering"
new_document appendParagraph "\tThis is again level 2 of numbering" [eval list [format $numberingStyle 1]]
new_document appendParagraph "\tThis is again level 3 of numbering" [eval list [format $numberingStyle 2]]
new_document appendParagraph "\tdecimal alignment\t3,1\tright aligned"
set handle [new_document appendParagraph "\tdecimal alignment\t123,1\talso right aligned"]
new_document appendParagraph "\tThis is again numbering level 1" [eval list [format $numberingStyle 0]]
new_document setNumberingStyleProperties $numberingStyleName [list 0 1 2] [list ParentNumbering 3]

catch {tcluno_soffice::windowTitle [new_document cget -desktop] "TCLUNO - DEMO: Simple Numbering"}
}; # END OF EXAMPLE-CODE

set name "Writer:TextTable in Writer"
set description($name) {
This example creates a text table within a Writer document with "normal" text above and below
}; # END OF EXAMPLE-DESCRIPTION

set example($name) {
package require itcluno 0.2

catch {itcl::delete object new_document}

itcluno::Writer new_document

new_document appendText "This text will be followed by a TextTable"
set textTable [new_document appendTable 5 3]
$textTable setPropertyValues [itcluno::OfficeUtilities::tkColor2ooColor BackColor "light blue"]
$textTable setCellValue [list 0 0] \
	[list \
		[list "Cell 0.0" "Cell 0.1" "Cell 0.2"] \
		[list "Cell 1.0" "Cell 1.1" "Cell 1.2"] \
		[list "Cell 2.0" "Cell 2.1" "Cell 2.2"] \
		[list "Cell 3.0" "Cell 3.1" "Cell 3.2"] \
		[list "Cell 4.0" "Cell 4.1" "Cell 4.2"] \
	]
new_document appendText "Here we go with more text"

catch {tcluno_soffice::windowTitle [new_document cget -desktop] "TCLUNO - DEMO: TextTable in Writer"}
}; # END OF EXAMPLE-CODE

set name "Writer:Formula in Writer"
set description($name) {
This example creates a formula within a Writer document
}; # END OF EXAMPLE-DESCRIPTION
set example($name) {
package require itcluno 0.2

catch {itcl::delete object new_document}

itcluno::Writer new_document

new_document appendParagraph "Here we have the famous wave equation:"
set formula [new_document appendFormula "{{partial^2}u} over {partial t^2} = c^2 %DELTA u"]
new_document appendParagraph "(as found on en.wikipedia.org/wiki/wave_equation)"
new_document appendParagraph "or n-dimensional:"
set formula [new_document appendFormula "{c^2{sum from i=1 to n left ({{partial^2}u} over {partial x_i^2} right )}} - {{partial^2}u} over {partial t^2} = 0"]
new_document appendText "(as found on de.wikipedia.org/wiki/Wellengleichung)"
new_document appendParagraph " " [list ParaAdjust 3]
set para [new_document appendParagraph "Formula Properties:" [list ParaStyleName "Heading 1"]]
array set properties [new_document getPropertyValues $formula]
#parray properties
set textTable [new_document appendTable [expr {[llength [array names properties]]+1}] 2]
set propertyList [list]
lappend propertyList [list Property Value]
foreach property [lsort [array names properties]] {
	lappend propertyList [list $property $properties($property)]
}
$textTable setCellValue [list 0 0] $propertyList

catch {tcluno_soffice::windowTitle [new_document cget -desktop] "TCLUNO - DEMO: Formula in Writer"}
}; # END OF EXAMPLE-CODE
